package jumbo.xml.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.io.IOException;

import java.net.URL;

import java.util.Hashtable;

import com.sun.java.swing.Icon;
import com.sun.java.swing.ImageIcon;
import com.sun.java.swing.JButton;
import com.sun.java.swing.JFrame;
import com.sun.java.swing.JLabel;
import com.sun.java.swing.JPanel;
import com.sun.java.swing.JTextPane;

import com.sun.java.swing.text.BadLocationException;
import com.sun.java.swing.text.DefaultStyledDocument;
import com.sun.java.swing.text.SimpleAttributeSet;
import com.sun.java.swing.text.StyleConstants;
import com.sun.java.swing.text.StyleContext;

/** a simple implemntation of a Swing text document.
/** (from Swing tutorial)
<PRE>
	Then, at the appropriate time, you can use the various methods of StyleConstants to
    change the style of the selected contents within the JTextPane or use
    StyledDocument methods like setCharacterAttributes,
    setParagraphAttributes, or just plain setLogicalStyle, to change the
    document characteristics. Just create a SimpleAttributeSet, and configure any
    attribute you would like:

                  void setAlignment()
                  void setBold()
                  void setComponent()
                  void setFirstLineIndent()
                  void setFontFamily()
                  void setFontSize()
                  void setForeground()
                  void setIcon()
                  void setItalic()
                  void setLeftIndent()
                  void setLineSpacing()
                  void setRightIndent()
                  void setSpaceAbove()
                  void setSpaceBelow()
                  void setUnderline()
                              StyleConstants methods for changing
                              attributes of currently selected content.
                              You will also need to
                              setCharacterAttributes or
                              setParagraphAttributes the
                              JTextPane.
                  void replaceSelection(String c)
                  void insertComponent(Component c)
                  void insertIcon(Icon g)
                         JTextPane methods to replace currently
                         selected content with a String,
                         Component, or Icon
                  void setLogicalStyle(Style s)
                         Changes Style of current paragraph
</PRE>
@author P.Murray-Rust, 1998
*/
public class XText extends DefaultStyledDocument {

// style context (for icons??)
	StyleContext styleContext = null;

// hashtable for line numbers
	Hashtable lineTable = null;

	JTextPane jTextPane = null;
/** default style 
*/
	public static final SimpleAttributeSet defaultStyle = 
		new SimpleAttributeSet();

// current style
	SimpleAttributeSet style = defaultStyle;

/** default constructor */
	public XText() {
		super();
		initialise();
		styleContext = new StyleContext();
	}

/** use existing style context (required for icons?)
*/
	public XText(StyleContext styleContext) {
		super(styleContext);
		this.styleContext = styleContext;
		initialise();
	}

/** create from a URL
*/
	public XText(URL url) throws IOException {
		this();
		lineTable = new Hashtable();
		InputStream is = url.openStream();
		BufferedReader br = new BufferedReader(new InputStreamReader(is));
// lines start from 1
		int lineCount = 0;
		while (true) {
			String s = br.readLine();
			if (s == null) break;
			int textLength = this.getLength();
			lineTable.put(new Integer(++lineCount), new Integer(textLength));
			this.addText(s+"\n");
		}
	}


	void initialise() {
		jTextPane = new JTextPane(this);
		jTextPane.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
//				System.out.println("Clicked "+jTextPane.getSelectionStart());
			}
		});
	}

/** make a SimpleAttributeSet (i.e. a style) from a Font. Note that the components
of a SimpleAttributeSet can be poked in individually if required).
@param Font font the font to set the style to
*/
	public static SimpleAttributeSet getSimpleAttributeSet(Font font) {
		SimpleAttributeSet sas = new SimpleAttributeSet();
		StyleConstants.setFontFamily(sas, font.getName());
		StyleConstants.setFontSize(sas, font.getSize());
		StyleConstants.setBold(sas, font.isBold());
		StyleConstants.setItalic(sas, font.isItalic());
		return sas;
	}

/** make a SimpleAttributeSet (i.e. a style) from a Color. Note that the components
of a SimpleAttributeSet can be poked in individually if required).
@param Font font the font to set the style to
*/
	public static SimpleAttributeSet getSimpleAttributeSet(Font font, Color foreground) {
		SimpleAttributeSet sas = new SimpleAttributeSet();
		StyleConstants.setForeground(sas, foreground);
		StyleConstants.setFontFamily(sas, font.getName());
		StyleConstants.setFontSize(sas, font.getSize());
		StyleConstants.setBold(sas, font.isBold());
		StyleConstants.setItalic(sas, font.isItalic());
		return sas;
	}

/** return a JTextPane containing the document. At present this is the primary 
way of using this class
@return JTextPane contains the rendered document
*/
	public JTextPane getJTextPane() {
		return jTextPane;
	}

/** add text using current style
@param String text text to be added (newlines have to be explicitly included)
@param SimpleAttributeSet style the style (see Swing documentation for how to create this)
*/
	public void addText(String text, SimpleAttributeSet style) {
		try {
			this.insertString (this.getLength(), text, style);
			this.style = style;
		} catch (Exception e) {System.out.println("BUG: "+e);}
	}

/** get  current text
*/
	public String getText() {
		int l = this.getLength();
		String s = null;
		try {
			s = this.getText(0, l);
		} catch (Exception e) {}
		return s;
	}

/** add text using current style
@param String text text to be added
*/
	public void addText(String text) {
		this.addText(text, style);
	}

/** add a component
@param Component component to be added
*/
	public void addComponent(Component component) {
		jTextPane.insertComponent(component);
	}

/** add icon (only way so far I can add an image). It doesn't work yet
@param Icon image to be added
*/
	public void addIcon(Icon icon) {
//		int l = this.getLength();
//		addText(" ");
//		jTextPane.setSelectionStart(l);
//		jTextPane.setSelectionEnd(l);
		jTextPane.insertIcon(icon);
	}

/** display the Text in a JFrame*/
	public void displayInFrame() {
		JFrame jFrame = new JFrame("XTextTest");
		jFrame.getContentPane().add(jTextPane);
		jFrame.pack();
		jFrame.show();
		jFrame.setSize(400,400);
	}

/** highlight by line and column; BOTH start from 1 (ONE)*/
	public void highlight(int line, int column) {
		if (lineTable == null) return;
		if (line <= 0 || line > lineTable.size()) return;
		if (column <= 0) column = 1;
		int len = ((Integer) lineTable.get(new Integer(line))).intValue();
		len += column-1;
		jTextPane.setSelectionStart(len);
		jTextPane.setSelectionEnd(++len);
	}

/** make the JTextPane (un)editable
@param boolean editable 
*/
	public void setEditable(boolean editable) {
		jTextPane.setEditable(editable);
	}

/** simple test routine */
	public static void main(String[] args) {
		if (args.length == 0) {
			System.out.println("Usage: java  XText foo");
			System.exit(1);
		}
		try {
			XText xText	= new XText();
	        SimpleAttributeSet defaultStyle = new SimpleAttributeSet();
			SimpleAttributeSet italicStyle = XText.getSimpleAttributeSet(new Font("Helvetica", Font.ITALIC, 10));
			SimpleAttributeSet bigStyle = XText.getSimpleAttributeSet(new Font("TimesRoman", Font.BOLD, 24));
			SimpleAttributeSet redStyle = XText.getSimpleAttributeSet(new Font("Helvetica", Font.BOLD, 14), Color.red);
			JTextPane jTextPane = xText.getJTextPane();
			jTextPane.setSelectionColor(Color.magenta);
			xText.addText("This is an XText", defaultStyle);
			int start = xText.getLength();
			xText.addText("second Text\n", bigStyle);
// must set selection after the next text has been inserted
			jTextPane.setSelectionStart(start);
			int end = xText.getLength();
			xText.addText("next para", italicStyle);
			jTextPane.setSelectionEnd(end);
			ImageIcon imageIcon = new ImageIcon("blue-ball.gif");
//			xText.addIcon(imageIcon);
			xText.addText("next default", redStyle);
			xText.addText(" and next bit", defaultStyle);

			JButton jButton = new JButton("Press");
//			xText.addComponent(jButton);

			xText.addText(" and more bits", italicStyle);
			imageIcon = new ImageIcon("red-ball.gif");
//			xText.addIcon(imageIcon);
			xText.addText(" and yet more bits", defaultStyle);
			xText.addText(" and the end bit", defaultStyle);
			xText.displayInFrame();
			System.out.println("Content: "+xText.getText());
		} catch (Exception e) {e.printStackTrace();}
	}
}
